@@ -3,5 +3,5 @@
 """Top-level package for PGSync."""
 
 __author__ = "Tolu Aina"
-__email__ = "toluaina@hotmail.com"
-__version__ = "2.3.2"
+__email__ = "tolu@pgsync.com"
+__version__ = "2.3.3"
