@@ -569,7 +569,7 @@ class TestParentSingleChildFkOnChild(object):
             [sort_list(doc) for doc in sync.sync()]
         msg = (
             "No foreign key relationship between "
-            '"public.book" and "public.city"'
+            "public.book and public.city"
         )
         assert msg in str(excinfo.value)
 
